class ConfigKey(object):
    LOCATOR_ADDRESS = "locatorAddress"
    PUB_ADDRESS = "pubAddress"
    SUB_ADDRESS = "subAddress"
    PULL_ADDRESS = "pullAddress"
    MIN_STREAMER_PORT = "minStreamerPort"
    MAX_STREAMER_PORT = "maxStreamerPort"
    PUBLIC_HOSTNAME = "publicHostname"
    STREAMER_POLL_TIMEOUT = "streamerPollTimeout"
