from messagebus import TokenCreator, TokenPriority

event_manager_group = "2"

TK_EVENT_MANAGER_UNSUBSCRIBE = TokenCreator.create_token(TokenPriority.low, event_manager_group, "2")
TK_EVENT_MANAGER_PUBLISH = TokenCreator.create_token(TokenPriority.low, event_manager_group, "3")
TK_EVENT_MANAGER_KEEP_ALIVE = TokenCreator.create_token(TokenPriority.low, event_manager_group, "4")
